@@ -253,6 +253,149 @@ def get_hdf5_with_external_recursive_links():
     return tmp1.name
 
 
+def get_hdf5_with_nxdata():
+    global _file_cache
+    ID = "nxdata"
+    if ID in _file_cache:
+        return _file_cache[ID].name
+
+    tmp = tempfile.NamedTemporaryFile(prefix=ID + "_", suffix=".h5", delete=True)
+    tmp.file.close()
+    h5 = h5py.File(tmp.name, "w")
+
+    # SCALARS
+    g0d = h5.create_group("scalars")
+
+    g0d0 = g0d.create_group("0D_scalar")
+    g0d0.attrs["NX_class"] = "NXdata"
+    g0d0.attrs["signal"] = "scalar"
+    g0d0.create_dataset("scalar", data=10)
+
+    g0d1 = g0d.create_group("2D_scalars")
+    g0d1.attrs["NX_class"] = "NXdata"
+    g0d1.attrs["signal"] = "scalars"
+    ds = g0d1.create_dataset("scalars", data=numpy.arange(3*10).reshape((3, 10)))
+    ds.attrs["interpretation"] = "scalar"
+
+    g0d1 = g0d.create_group("4D_scalars")
+    g0d1.attrs["NX_class"] = "NXdata"
+    g0d1.attrs["signal"] = "scalars"
+    ds = g0d1.create_dataset("scalars", data=numpy.arange(2*2*3*10).reshape((2, 2, 3, 10)))
+    ds.attrs["interpretation"] = "scalar"
+
+    # SPECTRA
+    g1d = h5.create_group("spectra")
+
+    g1d0 = g1d.create_group("1D_spectrum")
+    g1d0.attrs["NX_class"] = "NXdata"
+    g1d0.attrs["signal"] = "count"
+    g1d0.attrs["axes"] = "energy_calib"
+    g1d0.attrs["uncertainties"] = b"energy_errors",
+    g1d0.create_dataset("count", data=numpy.arange(10))
+    g1d0.create_dataset("energy_calib", data=(10, 5))     # 10 * idx + 5
+    g1d0.create_dataset("energy_errors", data=3.14*numpy.random.rand(10))
+
+    g1d1 = g1d.create_group("2D_spectra")
+    g1d1.attrs["NX_class"] = "NXdata"
+    g1d1.attrs["signal"] = "counts"
+    ds = g1d1.create_dataset("counts", data=numpy.arange(3*10).reshape((3, 10)))
+    ds.attrs["interpretation"] = "spectrum"
+
+    g1d2 = g1d.create_group("4D_spectra")
+    g1d2.attrs["NX_class"] = "NXdata"
+    g1d2.attrs["signal"] = "counts"
+    g1d2.attrs["axes"] = b"energy",
+    ds = g1d2.create_dataset("counts", data=numpy.arange(2*2*3*10).reshape((2, 2, 3, 10)))
+    ds.attrs["interpretation"] = "spectrum"
+    ds = g1d2.create_dataset("errors", data=4.5*numpy.random.rand(2, 2, 3, 10))
+    ds = g1d2.create_dataset("energy", data=5+10*numpy.arange(15),
+                             shuffle=True, compression="gzip")
+    ds.attrs["long_name"] = "Calibrated energy"
+    ds.attrs["first_good"] = 3
+    ds.attrs["last_good"] = 12
+    g1d2.create_dataset("energy_errors", data=10*numpy.random.rand(15))
+
+    # IMAGES
+    g2d = h5.create_group("images")
+
+    g2d0 = g2d.create_group("2D_regular_image")
+    g2d0.attrs["NX_class"] = "NXdata"
+    g2d0.attrs["signal"] = "image"
+    g2d0.attrs["axes"] = b"rows_calib", b"columns_coordinates"
+    g2d0.create_dataset("image", data=numpy.arange(4*6).reshape((4, 6)))
+    ds = g2d0.create_dataset("rows_calib", data=(10, 5))
+    ds.attrs["long_name"] = "Calibrated Y"
+    g2d0.create_dataset("columns_coordinates", data=0.5+0.02*numpy.arange(6))
+
+    g2d1 = g2d.create_group("2D_irregular_data")
+    g2d1.attrs["NX_class"] = "NXdata"
+    g2d1.attrs["signal"] = "data"
+    g2d1.attrs["axes"] = b"rows_coordinates", b"columns_coordinates"
+    g2d1.create_dataset("data", data=numpy.arange(64*128).reshape((64, 128)))
+    g2d1.create_dataset("rows_coordinates", data=numpy.arange(64) + numpy.random.rand(64))
+    g2d1.create_dataset("columns_coordinates", data=numpy.arange(128) + 2.5 * numpy.random.rand(128))
+
+    g2d2 = g2d.create_group("3D_images")
+    g2d2.attrs["NX_class"] = "NXdata"
+    g2d2.attrs["signal"] = "images"
+    ds = g2d2.create_dataset("images", data=numpy.arange(2*4*6).reshape((2, 4, 6)))
+    ds.attrs["interpretation"] = "image"
+
+    g2d3 = g2d.create_group("5D_images")
+    g2d3.attrs["NX_class"] = "NXdata"
+    g2d3.attrs["signal"] = "images"
+    g2d3.attrs["axes"] = b"rows_coordinates", b"columns_coordinates"
+    ds = g2d3.create_dataset("images", data=numpy.arange(2*2*2*4*6).reshape((2, 2, 2, 4, 6)))
+    ds.attrs["interpretation"] = "image"
+    g2d3.create_dataset("rows_coordinates", data=5+10*numpy.arange(4))
+    g2d3.create_dataset("columns_coordinates", data=0.5+0.02*numpy.arange(6))
+
+    # SCATTER
+    g = h5.create_group("scatters")
+
+    gd0 = g.create_group("x_y_scatter")
+    gd0.attrs["NX_class"] = "NXdata"
+    gd0.attrs["signal"] = "y"
+    gd0.attrs["axes"] = b"x",
+    gd0.create_dataset("y", data=numpy.random.rand(128) - 0.5)
+    gd0.create_dataset("x", data=2*numpy.random.rand(128))
+    gd0.create_dataset("x_errors", data=0.05*numpy.random.rand(128))
+    gd0.create_dataset("errors", data=0.05*numpy.random.rand(128))
+
+    gd1 = g.create_group("x_y_value_scatter")
+    gd1.attrs["NX_class"] = "NXdata"
+    gd1.attrs["signal"] = "values"
+    gd1.attrs["axes"] = b"x", b"y"
+    gd1.create_dataset("values", data=3.14*numpy.random.rand(128))
+    gd1.create_dataset("y", data=numpy.random.rand(128))
+    gd1.create_dataset("y_errors", data=0.02*numpy.random.rand(128))
+    gd1.create_dataset("x", data=numpy.random.rand(128))
+    gd1.create_dataset("x_errors", data=0.02*numpy.random.rand(128))
+
+    # NDIM > 3
+    g = h5.create_group("cubes")
+
+    gd0 = g.create_group("3D_cube")
+    gd0.attrs["NX_class"] = "NXdata"
+    gd0.attrs["signal"] = "cube"
+    gd0.attrs["axes"] = b"img_idx", b"rows_coordinates", b"cols_coordinates"
+    gd0.create_dataset("cube", data=numpy.arange(4*5*6).reshape((4, 5, 6)))
+    gd0.create_dataset("img_idx", data=numpy.arange(4))
+    gd0.create_dataset("rows_coordinates", data=0.1*numpy.arange(5))
+    gd0.create_dataset("cols_coordinates", data=[0.2, 0.3])  # linear calibration
+
+    gd1 = g.create_group("5D")
+    gd1.attrs["NX_class"] = "NXdata"
+    gd1.attrs["signal"] = "hypercube"
+    gd1.create_dataset("hypercube",
+                       data=numpy.arange(2*3*4*5*6).reshape((2, 3, 4, 5, 6)))
+
+    h5.close()
+
+    _file_cache[ID] = tmp
+    return tmp.name
+
+
 def get_edf_with_all_types():
     global _file_cache
     ID = "alltypesedf"
@@ -496,6 +639,7 @@ class Hdf5TreeViewExample(qt.QMainWindow):
         combo.addItem("Containing 100000 datasets", get_hdf5_with_100000_datasets)
         combo.addItem("Containing recursive links", get_hdf5_with_recursive_links)
         combo.addItem("Containing external recursive links", get_hdf5_with_external_recursive_links)
+        combo.addItem("Containing NXdata groups", get_hdf5_with_nxdata)
         combo.activated.connect(self.__hdf5ComboChanged)
         content.layout().addWidget(combo)
 
