@@ -29,7 +29,7 @@ This widget is meant to work with :class:`PlotWindow`.
 ROI are defined by :
 
 - A name (`ROI` column)
-- A type. The type is the label of the x axis. 
+- A type. The type is the label of the x axis.
   This can be used to apply or not some ROI to a curve and do some post processing.
 - The x coordinate of the left limit (`from` column)
 - The x coordinate of the right limit (`to` column)
@@ -46,8 +46,9 @@ ROI are defined by :
 
 __authors__ = ["V.A. Sole", "T. Vincent"]
 __license__ = "MIT"
-__date__ = "19/12/2016"
+__date__ = "27/06/2017"
 
+from collections import OrderedDict
 
 import logging
 import os
@@ -161,6 +162,58 @@ class CurvesROIWidget(qt.QWidget):
     def roiFileDir(self, roiFileDir):
         self._roiFileDir = str(roiFileDir)
 
+    def setRois(self, roidict, order=None):
+        """Set the ROIs by providing a dictionary of ROI information.
+
+        The dictionary keys are the ROI names.
+        Each value is a sub-dictionary of ROI info with the following fields:
+
+        - ``"from"``: x coordinate of the left limit, as a float
+        - ``"to"``: x coordinate of the right limit, as a float
+        - ``"type"``: type of ROI, as a string (e.g "channels", "energy")
+
+
+        :param roidict: Dictionary of ROIs
+        :param str order: Field used for ordering the ROIs.
+             One of "from", "to", "type".
+             None (default) for no ordering, or same order as specified
+             in parameter ``roidict`` if provided as an OrderedDict.
+        """
+        if order is None or order.lower() == "none":
+            roilist = list(roidict.keys())
+        else:
+            assert order in ["from", "to", "type"]
+            roilist = sorted(roidict.keys(),
+                             key=lambda roi_name: roidict[roi_name].get(order))
+
+        return self.roiTable.fillFromROIDict(roilist, roidict)
+
+    def getRois(self, order=None):
+        """Return the currently defined ROIs, as an ordered dict.
+
+        The dictionary keys are the ROI names.
+        Each value is a sub-dictionary of ROI info with the following fields:
+
+        - ``"from"``: x coordinate of the left limit, as a float
+        - ``"to"``: x coordinate of the right limit, as a float
+        - ``"type"``: type of ROI, as a string (e.g "channels", "energy")
+
+
+        :param order: Field used for ordering the ROIs.
+             One of "from", "to", "type", "netcounts", "rawcounts".
+             None (default) to get the same order as displayed in the widget.
+        :return: Ordered dictionary of ROI information
+        """
+        roilist, roidict = self.roiTable.getROIListAndDict()
+        if order is None or order.lower() == "none":
+            ordered_roilist = roilist
+        else:
+            assert order in ["from", "to", "type", "netcounts", "rawcounts"]
+            ordered_roilist = sorted(roidict.keys(),
+                                     key=lambda roi_name: roidict[roi_name].get(order))
+
+        return OrderedDict([(name, roidict[name]) for name in ordered_roilist])
+
     def _add(self):
         """Add button clicked handler"""
         ddict = {}
@@ -368,7 +421,18 @@ class ROITable(qt.QTableWidget):
             between the segment [maxPt, minPt] and the selected curve')
 
     def fillFromROIDict(self, roilist=(), roidict=None, currentroi=None):
-        """Set the ROIs
+        """Set the ROIs by providing a list of ROI names and a dictionary
+        of ROI information for each ROI.
+
+        The ROI names must match an existing dictionary key.
+        The name list is used to provide an order for the ROIs.
+
+        The dictionary's values are sub-dictionaries containing 3
+        mandatory fields:
+
+           - ``"from"``: x coordinate of the left limit, as a float
+           - ``"to"``: x coordinate of the right limit, as a float
+           - ``"type"``: type of ROI, as a string (e.g "channels", "energy")
 
         :param roilist: List of ROI names (keys of roidict)
         :type roilist: List
@@ -447,10 +511,24 @@ class ROITable(qt.QTableWidget):
         self.building = False
 
     def getROIListAndDict(self):
-        """Return the currently defined ROIs
+        """Return the currently defined ROIs, as a 2-tuple
+        ``(roiList, roiDict)``
 
-        :return: ROIs information
-        :rtype: ordered dict as a tuple of (list of ROI names, dict of info)
+        ``roiList`` is a list of ROI names.
+        ``roiDict`` is a dictionary of ROI info.
+
+        The ROI names must match an existing dictionary key.
+        The name list is used to provide an order for the ROIs.
+
+        The dictionary's values are sub-dictionaries containing 3
+        fields:
+
+           - ``"from"``: x coordinate of the left limit, as a float
+           - ``"to"``: x coordinate of the right limit, as a float
+           - ``"type"``: type of ROI, as a string (e.g "channels", "energy")
+
+
+        :return: ordered dict as a tuple of (list of ROI names, dict of info)
         """
         return self.roilist, self.roidict
 
@@ -537,7 +615,7 @@ class ROITable(qt.QTableWidget):
 class CurvesROIDockWidget(qt.QDockWidget):
     """QDockWidget with a :class:`CurvesROIWidget` connected to a PlotWindow.
 
-    It makes the link between the CurvesROIWidget and the PlotWindow.
+    It makes the link between the :class:`CurvesROIWidget` and the PlotWindow.
 
     :param parent: See :class:`QDockWidget`
     :param plot: :class:`.PlotWindow` instance on which to operate
@@ -558,6 +636,14 @@ class CurvesROIDockWidget(qt.QDockWidget):
         self._isInit = False
 
         self.roiWidget = CurvesROIWidget(self, name)
+        """Main widget of type :class:`CurvesROIWidget`"""
+
+        # convenience methods to offer a simpler API allowing to ignore
+        # the details of the underlying implementation
+        self.calculateROIs = self.calculateRois
+        self.setRois = self.roiWidget.setRois
+        self.getRois = self.roiWidget.getRois
+
         self.layout().setContentsMargins(0, 0, 0, 0)
         self.setWidget(self.roiWidget)
 
@@ -658,7 +744,7 @@ class CurvesROIDockWidget(qt.QDockWidget):
         """Handle ROI widget signal"""
         _logger.debug("PlotWindow._roiSignal %s", str(ddict))
         if ddict['event'] == "AddROI":
-            xmin, xmax = self.plot.getGraphXLimits()
+            xmin, xmax = self.plot.getXAxis().getLimits()
             fromdata = xmin + 0.25 * (xmax - xmin)
             todata = xmin + 0.75 * (xmax - xmin)
             self.plot.remove('ROI min', kind='marker')
@@ -702,7 +788,7 @@ class CurvesROIDockWidget(qt.QDockWidget):
             if newroi == "ICR":
                 roiDict[newroi]['type'] = "Default"
             else:
-                roiDict[newroi]['type'] = self.plot.getGraphXLabel()
+                roiDict[newroi]['type'] = self.plot.getXAxis().getLabel()
             roiDict[newroi]['from'] = fromdata
             roiDict[newroi]['to'] = todata
             self.roiWidget.fillFromROIDict(roilist=roiList,
@@ -782,7 +868,7 @@ class CurvesROIDockWidget(qt.QDockWidget):
         """Recompute ROIs when active curve changed."""
         self.calculateROIs()
 
-    def calculateROIs(self, roiList=None, roiDict=None):
+    def calculateRois(self, roiList=None, roiDict=None):
         """Compute ROI information"""
         if roiList is None or roiDict is None:
             roiList, roiDict = self.roiWidget.getROIListAndDict()
@@ -790,11 +876,15 @@ class CurvesROIDockWidget(qt.QDockWidget):
         activeCurve = self.plot.getActiveCurve(just_legend=False)
         if activeCurve is None:
             xproc = None
+            yproc = None
             self.roiWidget.setHeader()
         else:
-            x, y, legend = activeCurve[0:3]
+            x = activeCurve.getXData(copy=False)
+            y = activeCurve.getYData(copy=False)
+            legend = activeCurve.getLegend()
             idx = numpy.argsort(x, kind='mergesort')
             xproc = numpy.take(x, idx)
+            yproc = numpy.take(y, idx)
             self.roiWidget.setHeader('ROIs of %s' % legend)
 
         for key in roiList:
@@ -811,8 +901,8 @@ class CurvesROIDockWidget(qt.QDockWidget):
                 idx = numpy.nonzero((fromData <= xproc) &
                                     (xproc <= toData))[0]
                 if len(idx):
-                    xw = x[idx]
-                    yw = y[idx]
+                    xw = xproc[idx]
+                    yw = yproc[idx]
                     rawCounts = yw.sum(dtype=numpy.float)
                     deltaX = xw[-1] - xw[0]
                     deltaY = yw[-1] - yw[0]
@@ -858,7 +948,8 @@ class CurvesROIDockWidget(qt.QDockWidget):
         xmax, ymax = None, None
 
         for curve in curves:
-            x, y = curve[0:2]
+            x = curve.getXData(copy=False)
+            y = curve.getYData(copy=False)
             if xmin is None:
                 xmin = x.min()
             else:
@@ -877,3 +968,10 @@ class CurvesROIDockWidget(qt.QDockWidget):
                 ymax = max(ymax, y.max())
 
         return xmin, ymin, xmax, ymax
+
+    def showEvent(self, event):
+        """Make sure this widget is raised when it is shown
+        (when it is first created as a tab in PlotWindow or when it is shown
+        again after hiding).
+        """
+        self.raise_()
