@@ -26,7 +26,7 @@
 
 __authors__ = ["T. Vincent"]
 __license__ = "MIT"
-__date__ = "24/01/2017"
+__date__ = "27/06/2017"
 
 
 import math
@@ -35,14 +35,15 @@ import time
 import weakref
 
 from . import Colors
+from . import items
 from .Interaction import (ClickOrDrag, LEFT_BTN, RIGHT_BTN,
                           State, StateMachine)
 from .PlotEvents import (prepareCurveSignal, prepareDrawingSignal,
                          prepareHoverSignal, prepareImageSignal,
                          prepareMarkerSignal, prepareMouseSignal)
 
-from .BackendBase import (CURSOR_POINTING, CURSOR_SIZE_HOR,
-                          CURSOR_SIZE_VER, CURSOR_SIZE_ALL)
+from .backends.BackendBase import (CURSOR_POINTING, CURSOR_SIZE_HOR,
+                                   CURSOR_SIZE_VER, CURSOR_SIZE_ALL)
 
 from ._utils import (FLOAT32_SAFE_MIN, FLOAT32_MINPOS, FLOAT32_SAFE_MAX,
                      applyZoomToPlot)
@@ -76,7 +77,7 @@ class _PlotInteraction(object):
 
         :param points: The 2D coordinates of the points of the polygon
         :type points: An iterable of (x, y) coordinates
-        :param str fill: The fill mode: 'hatch', 'solid' or None
+        :param str fill: The fill mode: 'hatch', 'solid' or 'none'
         :param color: RGBA color to use or None to disable display
         :type color: list or tuple of 4 float in the range [0, 1]
         :param name: The key associated with this selection area
@@ -92,7 +93,7 @@ class _PlotInteraction(object):
         # TODO Not very nice, but as is for now
         legend = '__SELECTION_AREA__' + name
 
-        fill = bool(fill)  # TODO not very nice either
+        fill = fill != 'none'  # TODO not very nice either
 
         self.plot.addItem(points[:, 0], points[:, 1], legend=legend,
                           replace=False,
@@ -152,11 +153,11 @@ class Pan(_ZoomOnWheel):
         xData, yData, y2Data = self._pixelToData(x, y)
         lastX, lastY, lastY2 = self._previousDataPos
 
-        xMin, xMax = self.plot.getGraphXLimits()
-        yMin, yMax = self.plot.getGraphYLimits(axis='left')
-        y2Min, y2Max = self.plot.getGraphYLimits(axis='right')
+        xMin, xMax = self.plot.getXAxis().getLimits()
+        yMin, yMax = self.plot.getYAxis().getLimits()
+        y2Min, y2Max = self.plot.getYAxis(axis='right').getLimits()
 
-        if self.plot.isXAxisLogarithmic():
+        if self.plot.getXAxis()._isLogarithmic():
             try:
                 dx = math.log10(xData) - math.log10(lastX)
                 newXMin = pow(10., (math.log10(xMin) - dx))
@@ -175,7 +176,7 @@ class Pan(_ZoomOnWheel):
             if newXMin < FLOAT32_SAFE_MIN or newXMax > FLOAT32_SAFE_MAX:
                 newXMin, newXMax = xMin, xMax
 
-        if self.plot.isYAxisLogarithmic():
+        if self.plot.getYAxis()._isLogarithmic():
             try:
                 dy = math.log10(yData) - math.log10(lastY)
                 newYMin = pow(10., math.log10(yMin) - dy)
@@ -232,10 +233,10 @@ class Zoom(_ZoomOnWheel):
 
     def __init__(self, plot, color):
         self.color = color
-        self.zoomStack = []
         self._lastClick = 0., None
 
         super(Zoom, self).__init__(plot)
+        self.plot.getLimitsHistory().clear()
 
     def _areaWithAspectRatio(self, x0, y0, x1, y1):
         _plotLeft, _plotTop, plotW, plotH = self.plot.getPlotBoundsInPixels()
@@ -285,25 +286,14 @@ class Zoom(_ZoomOnWheel):
 
                 self._lastClick = time.time(), (dataPos[0], dataPos[1], x, y)
 
-            # Zoom-in centered on mouse cursor
-            # xMin, xMax = self.plot.getGraphXLimits()
-            # yMin, yMax = self.plot.getGraphYLimits()
-            # y2Min, y2Max = self.plot.getGraphYLimits(axis="right")
-            # self.zoomStack.append((xMin, xMax, yMin, yMax, y2Min, y2Max))
-            # self._zoom(x, y, 2)
         elif btn == RIGHT_BTN:
-            try:
-                xMin, xMax, yMin, yMax, y2Min, y2Max = self.zoomStack.pop()
-            except IndexError:
-                # Signal mouse clicked event
-                dataPos = self.plot.pixelToData(x, y)
-                assert dataPos is not None
-                eventDict = prepareMouseSignal('mouseClicked', 'right',
-                                               dataPos[0], dataPos[1],
-                                               x, y)
-                self.plot.notify(**eventDict)
-            else:
-                self.plot.setLimits(xMin, xMax, yMin, yMax, y2Min, y2Max)
+            # Signal mouse clicked event
+            dataPos = self.plot.pixelToData(x, y)
+            assert dataPos is not None
+            eventDict = prepareMouseSignal('mouseClicked', 'right',
+                                           dataPos[0], dataPos[1],
+                                           x, y)
+            self.plot.notify(**eventDict)
 
     def beginDrag(self, x, y):
         dataPos = self.plot.pixelToData(x, y)
@@ -334,7 +324,7 @@ class Zoom(_ZoomOnWheel):
                 areaColor = [1., 1., 1., 1.]
 
             self.setSelectionArea(areaPoints,
-                                  fill=None,
+                                  fill='none',
                                   color=areaColor,
                                   name="zoomedArea")
 
@@ -345,7 +335,7 @@ class Zoom(_ZoomOnWheel):
         corners = numpy.array([self.plot.pixelToData(x, y, check=False)
                                for (x, y) in corners])
 
-        self.setSelectionArea(corners, fill=None, color=self.color)
+        self.setSelectionArea(corners, fill='none', color=self.color)
 
     def endDrag(self, startPos, endPos):
         x0, y0 = startPos
@@ -353,10 +343,7 @@ class Zoom(_ZoomOnWheel):
 
         if x0 != x1 or y0 != y1:  # Avoid empty zoom area
             # Store current zoom state in stack
-            xMin, xMax = self.plot.getGraphXLimits()
-            yMin, yMax = self.plot.getGraphYLimits()
-            y2Min, y2Max = self.plot.getGraphYLimits(axis="right")
-            self.zoomStack.append((xMin, xMax, yMin, yMax, y2Min, y2Max))
+            self.plot.getLimitsHistory().push()
 
             if self.plot.isKeepDataAspectRatio():
                 x0, y0, x1, y1 = self._areaWithAspectRatio(x0, y0, x1, y1)
@@ -509,33 +496,6 @@ class SelectPolygon(Select):
                     self.points[-1] = dataPos
 
                 return True
-
-            elif btn == RIGHT_BTN:
-                self.machine.resetSelectionArea()
-
-                firstPos = self.machine.plot.dataToPixel(*self._firstPos,
-                                                         check=False)
-                dx, dy = abs(firstPos[0] - x), abs(firstPos[1] - y)
-
-                if (dx < self.machine.DRAG_THRESHOLD_DIST and
-                        dy < self.machine.DRAG_THRESHOLD_DIST):
-                    self.points[-1] = self.points[0]
-                else:
-                    dataPos = self.machine.plot.pixelToData(x, y)
-                    assert dataPos is not None
-                    self.points[-1] = dataPos
-                    if self.points[-2] == self.points[-1]:
-                        self.points.pop()
-                    self.points.append(self.points[0])
-
-                eventDict = prepareDrawingSignal('drawingFinished',
-                                                 'polygon',
-                                                 self.points,
-                                                 self.machine.parameters)
-                self.machine.plot.notify(**eventDict)
-                self.goto('idle')
-                return False
-
             return False
 
         def onMove(self, x, y):
@@ -880,7 +840,7 @@ class DrawFreeHand(Select):
 
         polygon = center + self._circle
 
-        self.setSelectionArea(polygon, fill='', color=self.color)
+        self.setSelectionArea(polygon, fill='none', color=self.color)
 
     def select(self, x, y):
         pos = self.plot.pixelToData(x, y, check=False)
@@ -973,7 +933,7 @@ class SelectFreeLine(ClickOrDrag, _PlotInteraction):
             self.plot.notify(**eventDict)
 
         if not isLast:
-            self.setSelectionArea(self._points, fill=None, color=self.color,
+            self.setSelectionArea(self._points, fill='none', color=self.color,
                                   shape='polylines')
         else:
             self.cancel()
@@ -1004,10 +964,10 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
                 dataPos = self.machine.plot.pixelToData(x, y)
                 assert dataPos is not None
                 eventDict = prepareHoverSignal(
-                    marker['legend'], 'marker',
+                    marker.getLegend(), 'marker',
                     dataPos, (x, y),
-                    marker['draggable'],
-                    marker['selectable'])
+                    marker.isDraggable(),
+                    marker.isSelectable())
                 self.machine.plot.notify(**eventDict)
 
             if marker != self._hoverMarker:
@@ -1016,15 +976,15 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
                 if marker is None:
                     self.machine.plot.setGraphCursorShape()
 
-                elif marker['draggable']:
-                    if marker['x'] is None:
+                elif marker.isDraggable():
+                    if isinstance(marker, items.YMarker):
                         self.machine.plot.setGraphCursorShape(CURSOR_SIZE_VER)
-                    elif marker['y'] is None:
+                    elif isinstance(marker, items.XMarker):
                         self.machine.plot.setGraphCursorShape(CURSOR_SIZE_HOR)
                     else:
                         self.machine.plot.setGraphCursorShape(CURSOR_SIZE_ALL)
 
-                elif marker['selectable']:
+                elif marker.isSelectable():
                     self.machine.plot.setGraphCursorShape(CURSOR_POINTING)
 
             return True
@@ -1072,9 +1032,9 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
 
         if btn == LEFT_BTN:
             marker = self.plot._pickMarker(
-                x, y, lambda marker: marker['selectable'])
+                x, y, lambda m: m.isSelectable())
             if marker is not None:
-                xData, yData = marker['x'], marker['y']
+                xData, yData = marker.getPosition()
                 if xData is None:
                     xData = [0, 1]
                 if yData is None:
@@ -1082,31 +1042,36 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
 
                 eventDict = prepareMarkerSignal('markerClicked',
                                                 'left',
-                                                marker['legend'],
+                                                marker.getLegend(),
                                                 'marker',
-                                                marker['draggable'],
-                                                marker['selectable'],
+                                                marker.isDraggable(),
+                                                marker.isSelectable(),
                                                 (xData, yData),
                                                 (x, y), None)
                 return eventDict
 
             else:
                 picked = self.plot._pickImageOrCurve(
-                    x, y, lambda item: item['selectable'])
+                    x, y, lambda item: item.isSelectable())
 
                 if picked is None:
                     pass
 
                 elif picked[0] == 'curve':
                     curve = picked[1]
+                    indices = picked[2]
 
                     dataPos = self.plot.pixelToData(x, y)
                     assert dataPos is not None
 
+                    xData = curve.getXData(copy=False)
+                    yData = curve.getYData(copy=False)
+
                     eventDict = prepareCurveSignal('left',
-                                                   curve['legend'],
+                                                   curve.getLegend(),
                                                    'curve',
-                                                   picked[2], picked[3],
+                                                   xData[indices],
+                                                   yData[indices],
                                                    dataPos[0], dataPos[1],
                                                    x, y)
                     return eventDict
@@ -1118,13 +1083,12 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
                     assert dataPos is not None
 
                     # Get corresponding coordinate in image
-                    column = int((dataPos[0] - image['origin'][0]) /
-                                 float(image['scale'][0]))
-                    row = int((dataPos[1] - image['origin'][1]) /
-                              float(image['scale'][1]))
-
+                    origin = image.getOrigin()
+                    scale = image.getScale()
+                    column = int((dataPos[0] - origin[0]) / float(scale[0]))
+                    row = int((dataPos[1] - origin[1]) / float(scale[1]))
                     eventDict = prepareImageSignal('left',
-                                                   image['legend'],
+                                                   image.getLegend(),
                                                    'image',
                                                    column, row,
                                                    dataPos[0], dataPos[1],
@@ -1136,7 +1100,7 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
     def _signalMarkerMovingEvent(self, eventType, marker, x, y):
         assert marker is not None
 
-        xData, yData = marker['x'], marker['y']
+        xData, yData = marker.getPosition()
         if xData is None:
             xData = [0, 1]
         if yData is None:
@@ -1147,10 +1111,10 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
 
         eventDict = prepareMarkerSignal(eventType,
                                         'left',
-                                        marker['legend'],
+                                        marker.getLegend(),
                                         'marker',
-                                        marker['draggable'],
-                                        marker['selectable'],
+                                        marker.isDraggable(),
+                                        marker.isSelectable(),
                                         (xData, yData),
                                         (x, y),
                                         posDataCursor)
@@ -1169,23 +1133,24 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
         self.imageLegend = None
         self.markerLegend = None
         marker = self.plot._pickMarker(
-            x, y, lambda marker: marker['draggable'])
+            x, y, lambda m: m.isDraggable())
 
         if marker is not None:
-            self.markerLegend = marker['legend']
+            self.markerLegend = marker.getLegend()
             self._signalMarkerMovingEvent('markerMoving', marker, x, y)
         else:
             picked = self.plot._pickImageOrCurve(
                 x,
                 y,
-                lambda item: item.get('draggable', False))
+                lambda item:
+                    hasattr(item, 'isDraggable') and item.isDraggable())
             if picked is None:
                 self.imageLegend = None
                 self.plot.setGraphCursorShape()
                 return False
             else:
                 assert picked[0] == 'image'  # For now only drag images
-                self.imageLegend = picked[1]['legend']
+                self.imageLegend = picked[1].getLegend()
         return True
 
     def drag(self, x, y):
@@ -1195,24 +1160,25 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
 
         if self.markerLegend is not None:
             marker = self.plot._getMarker(self.markerLegend)
-            if marker['constraint'] is not None:
-                xData, yData = marker['constraint'](xData, yData)
+            if marker is not None:
+                marker.setPosition(xData, yData)
 
-            self.plot._moveMarker(self.markerLegend, xData, yData)
-
-            self._signalMarkerMovingEvent(
-                'markerMoving', self.plot._getMarker(self.markerLegend), x, y)
+                self._signalMarkerMovingEvent(
+                    'markerMoving', marker, x, y)
 
         if self.imageLegend is not None:
-            dx, dy = xData - self._lastPos[0], yData - self._lastPos[1]
-            self.plot._moveImage(self.imageLegend, dx, dy)
+            image = self.plot.getImage(self.imageLegend)
+            origin = image.getOrigin()
+            xImage = origin[0] + xData - self._lastPos[0]
+            yImage = origin[1] + yData - self._lastPos[1]
+            image.setOrigin((xImage, yImage))
 
         self._lastPos = xData, yData
 
     def endDrag(self, startPos, endPos):
         if self.markerLegend is not None:
             marker = self.plot._getMarker(self.markerLegend)
-            posData = [marker['x'], marker['y']]
+            posData = list(marker.getPosition())
             if posData[0] is None:
                 posData[0] = [0, 1]
             if posData[1] is None:
@@ -1221,10 +1187,10 @@ class ItemsInteraction(ClickOrDrag, _PlotInteraction):
             eventDict = prepareMarkerSignal(
                 'markerMoved',
                 'left',
-                marker['legend'],
+                marker.getLegend(),
                 'marker',
-                marker['draggable'],
-                marker['selectable'],
+                marker.isDraggable(),
+                marker.isSelectable(),
                 posData)
             self.plot.notify(**eventDict)
 
@@ -1272,7 +1238,7 @@ class FocusManager(StateMachine):
     class Focus(State):
         def enterState(self, eventHandler, btn):
             self.eventHandler = eventHandler
-            self.focusBtns = set((btn,))
+            self.focusBtns = {btn}
 
         def onPress(self, x, y, btn):
             self.focusBtns.add(btn)
