@@ -25,26 +25,30 @@
 """A :class:`.PlotWidget` with additional toolbars.
 
 The :class:`PlotWindow` is a subclass of :class:`.PlotWidget`.
-It provides the plot API fully defined in :class:`.Plot`.
 """
 
 __authors__ = ["V.A. Sole", "T. Vincent"]
 __license__ = "MIT"
-__date__ = "26/01/2017"
+__date__ = "17/08/2017"
 
 import collections
 import logging
 
-from silx.utils.decorators import deprecated
+from silx.utils.deprecation import deprecated
 
 from . import PlotWidget
-from . import PlotActions
+from . import actions
+from . import items
+from .actions import medfilt as actions_medfilt
+from .actions import fit as actions_fit
+from .actions import histogram as actions_histogram
 from . import PlotToolButtons
 from .PlotTools import PositionInfo
 from .Profile import ProfileToolBar
 from .LegendSelector import LegendsDockWidget
 from .CurvesROIWidget import CurvesROIDockWidget
 from .MaskToolsWidget import MaskToolsDockWidget
+from .ColorBar import ColorBarWidget
 try:
     from ..console import IPythonDockWidget
 except ImportError:
@@ -64,8 +68,8 @@ class PlotWindow(PlotWidget):
     Initialiser parameters:
 
     :param parent: The parent of this widget or None.
-    :param backend: The backend to use for the plot.
-                    The default is to use matplotlib.
+    :param backend: The backend to use for the plot (default: matplotlib).
+                    See :class:`.PlotWidget` for the list of supported backend.
     :type backend: str or :class:`BackendBase.BackendBase`
     :param bool resetzoom: Toggle visibility of reset zoom action.
     :param bool autoScale: Toggle visibility of axes autoscale actions.
@@ -113,46 +117,54 @@ class PlotWindow(PlotWidget):
         self.group = qt.QActionGroup(self)
         self.group.setExclusive(False)
 
-        self.resetZoomAction = self.group.addAction(PlotActions.ResetZoomAction(self))
+        self.zoomModeAction = self.group.addAction(
+            actions.mode.ZoomModeAction(self))
+        self.panModeAction = self.group.addAction(
+            actions.mode.PanModeAction(self))
+
+        self.resetZoomAction = self.group.addAction(
+            actions.control.ResetZoomAction(self))
         self.resetZoomAction.setVisible(resetzoom)
         self.addAction(self.resetZoomAction)
 
-        self.zoomInAction = PlotActions.ZoomInAction(self)
+        self.zoomInAction = actions.control.ZoomInAction(self)
         self.addAction(self.zoomInAction)
 
-        self.zoomOutAction = PlotActions.ZoomOutAction(self)
+        self.zoomOutAction = actions.control.ZoomOutAction(self)
         self.addAction(self.zoomOutAction)
 
         self.xAxisAutoScaleAction = self.group.addAction(
-            PlotActions.XAxisAutoScaleAction(self))
+            actions.control.XAxisAutoScaleAction(self))
         self.xAxisAutoScaleAction.setVisible(autoScale)
         self.addAction(self.xAxisAutoScaleAction)
 
         self.yAxisAutoScaleAction = self.group.addAction(
-            PlotActions.YAxisAutoScaleAction(self))
+            actions.control.YAxisAutoScaleAction(self))
         self.yAxisAutoScaleAction.setVisible(autoScale)
         self.addAction(self.yAxisAutoScaleAction)
 
         self.xAxisLogarithmicAction = self.group.addAction(
-            PlotActions.XAxisLogarithmicAction(self))
+            actions.control.XAxisLogarithmicAction(self))
         self.xAxisLogarithmicAction.setVisible(logScale)
         self.addAction(self.xAxisLogarithmicAction)
 
         self.yAxisLogarithmicAction = self.group.addAction(
-            PlotActions.YAxisLogarithmicAction(self))
+            actions.control.YAxisLogarithmicAction(self))
         self.yAxisLogarithmicAction.setVisible(logScale)
         self.addAction(self.yAxisLogarithmicAction)
 
         self.gridAction = self.group.addAction(
-            PlotActions.GridAction(self, gridMode='both'))
+            actions.control.GridAction(self, gridMode='both'))
         self.gridAction.setVisible(grid)
         self.addAction(self.gridAction)
 
-        self.curveStyleAction = self.group.addAction(PlotActions.CurveStyleAction(self))
+        self.curveStyleAction = self.group.addAction(
+            actions.control.CurveStyleAction(self))
         self.curveStyleAction.setVisible(curveStyle)
         self.addAction(self.curveStyleAction)
 
-        self.colormapAction = self.group.addAction(PlotActions.ColormapAction(self))
+        self.colormapAction = self.group.addAction(
+            actions.control.ColormapAction(self))
         self.colormapAction.setVisible(colormap)
         self.addAction(self.colormapAction)
 
@@ -171,26 +183,34 @@ class PlotWindow(PlotWidget):
         self.getMaskAction().setVisible(mask)
 
         self._intensityHistoAction = self.group.addAction(
-            PlotActions.PixelIntensitiesHistoAction(self))
+            actions_histogram.PixelIntensitiesHistoAction(self))
         self._intensityHistoAction.setVisible(False)
 
+        self._medianFilter2DAction = self.group.addAction(
+            actions_medfilt.MedianFilter2DAction(self))
+        self._medianFilter2DAction.setVisible(False)
+
+        self._medianFilter1DAction = self.group.addAction(
+            actions_medfilt.MedianFilter1DAction(self))
+        self._medianFilter1DAction.setVisible(False)
+
         self._separator = qt.QAction('separator', self)
         self._separator.setSeparator(True)
         self.group.addAction(self._separator)
 
-        self.copyAction = self.group.addAction(PlotActions.CopyAction(self))
+        self.copyAction = self.group.addAction(actions.io.CopyAction(self))
         self.copyAction.setVisible(copy)
         self.addAction(self.copyAction)
 
-        self.saveAction = self.group.addAction(PlotActions.SaveAction(self))
+        self.saveAction = self.group.addAction(actions.io.SaveAction(self))
         self.saveAction.setVisible(save)
         self.addAction(self.saveAction)
 
-        self.printAction = self.group.addAction(PlotActions.PrintAction(self))
+        self.printAction = self.group.addAction(actions.io.PrintAction(self))
         self.printAction.setVisible(print_)
         self.addAction(self.printAction)
 
-        self.fitAction = self.group.addAction(PlotActions.FitAction(self))
+        self.fitAction = self.group.addAction(actions_fit.FitAction(self))
         self.fitAction.setVisible(fit)
         self.addAction(self.fitAction)
 
@@ -199,6 +219,28 @@ class PlotWindow(PlotWidget):
         self._panWithArrowKeysAction = None
         self._crosshairAction = None
 
+        # Create color bar, hidden by default for backward compatibility
+        self._colorbar = ColorBarWidget(parent=self, plot=self)
+        self._colorbar.setVisible(False)
+
+        # Make colorbar background white
+        self._colorbar.setAutoFillBackground(True)
+        palette = self._colorbar.palette()
+        palette.setColor(qt.QPalette.Background, qt.Qt.white)
+        palette.setColor(qt.QPalette.Window, qt.Qt.white)
+        self._colorbar.setPalette(palette)
+
+        gridLayout = qt.QGridLayout()
+        gridLayout.setSpacing(0)
+        gridLayout.setContentsMargins(0, 0, 0, 0)
+        gridLayout.addWidget(self.getWidgetHandle(), 0, 0)
+        gridLayout.addWidget(self._colorbar, 0, 1)
+        gridLayout.setRowStretch(0, 1)
+        gridLayout.setColumnStretch(0, 1)
+        centralWidget = qt.QWidget()
+        centralWidget.setLayout(gridLayout)
+        self.setCentralWidget(centralWidget)
+
         if control or position:
             hbox = qt.QHBoxLayout()
             hbox.setContentsMargins(0, 0, 0, 0)
@@ -231,15 +273,7 @@ class PlotWindow(PlotWidget):
             bottomBar = qt.QWidget()
             bottomBar.setLayout(hbox)
 
-            layout = qt.QVBoxLayout()
-            layout.setSpacing(0)
-            layout.setContentsMargins(0, 0, 0, 0)
-            layout.addWidget(self.getWidgetHandle())
-            layout.addWidget(bottomBar)
-
-            centralWidget = qt.QWidget()
-            centralWidget.setLayout(layout)
-            self.setCentralWidget(centralWidget)
+            gridLayout.addWidget(bottomBar, 1, 0, 1, -1)
 
         # Creating the toolbar also create actions for toolbuttons
         self._toolbar = self._createToolBar(title='Plot', parent=None)
@@ -279,9 +313,9 @@ class PlotWindow(PlotWidget):
                 available_vars=available_vars,
                 custom_banner=banner,
                 parent=self)
-            self._introduceNewDockWidget(self._consoleDockWidget)
+            self.addTabbedDockWidget(self._consoleDockWidget)
             self._consoleDockWidget.visibilityChanged.connect(
-                    self.getConsoleAction().setChecked)
+                self.getConsoleAction().setChecked)
 
         self._consoleDockWidget.setVisible(is_checked)
 
@@ -313,6 +347,8 @@ class PlotWindow(PlotWidget):
                     self.yAxisInvertedAction = toolbar.addWidget(obj)
                 else:
                     raise RuntimeError()
+            if obj is self.panModeAction:
+                toolbar.addSeparator()
         return toolbar
 
     def toolBar(self):
@@ -344,9 +380,12 @@ class PlotWindow(PlotWidget):
         controlMenu.addAction(self.getCrosshairAction())
         controlMenu.addAction(self.getPanWithArrowKeysAction())
 
-    def _introduceNewDockWidget(self, dock_widget):
-        """Maintain a list of dock widgets, in the order in which they are
-        added. Tabify them as soon as there are more than one of them.
+    def addTabbedDockWidget(self, dock_widget):
+        """Add a dock widget as a new tab if there are already dock widgets
+        in the plot. When the first tab is added, the area is chosen
+        depending on the plot geometry:
+        it the window is much wider than it is high, the right dock area
+        is used, else the bottom dock area is used.
 
         :param dock_widget: Instance of :class:`QDockWidget` to be added.
         """
@@ -366,9 +405,16 @@ class PlotWindow(PlotWidget):
             self.tabifyDockWidget(self._dockWidgets[0],
                                   dock_widget)
 
+    def getColorBarWidget(self):
+        """Returns the embedded :class:`ColorBarWidget` widget.
+
+        :rtype: ColorBarWidget
+        """
+        return self._colorbar
+
     # getters for dock widgets
     @property
-    @deprecated
+    @deprecated(replacement="getLegendsDockWidget()", since_version="0.4.0")
     def legendsDockWidget(self):
         return self.getLegendsDockWidget()
 
@@ -377,25 +423,36 @@ class PlotWindow(PlotWidget):
         if self._legendsDockWidget is None:
             self._legendsDockWidget = LegendsDockWidget(plot=self)
             self._legendsDockWidget.hide()
-            self._introduceNewDockWidget(self._legendsDockWidget)
+            self.addTabbedDockWidget(self._legendsDockWidget)
         return self._legendsDockWidget
 
     @property
-    @deprecated
+    @deprecated(replacement="getCurvesRoiDockWidget()", since_version="0.4.0")
     def curvesROIDockWidget(self):
         return self.getCurvesRoiDockWidget()
 
     def getCurvesRoiDockWidget(self):
-        """DockWidget with curves' ROI panel (lazy-loaded)."""
+        """DockWidget with curves' ROI panel (lazy-loaded).
+
+        The widget returned is a :class:`CurvesROIDockWidget`.
+        Its central widget is a :class:`CurvesROIWidget`
+        accessible as :attr:`CurvesROIDockWidget.roiWidget`.
+
+        :class:`silx.gui.plot.CurvesROIWidget.CurvesROIWidget` offers a getter
+        and a setter for the ROI data:
+
+            - :meth:`CurvesROIWidget.getRois`
+            - :meth:`CurvesROIWidget.setRois`
+        """
         if self._curvesROIDockWidget is None:
             self._curvesROIDockWidget = CurvesROIDockWidget(
                 plot=self, name='Regions Of Interest')
             self._curvesROIDockWidget.hide()
-            self._introduceNewDockWidget(self._curvesROIDockWidget)
+            self.addTabbedDockWidget(self._curvesROIDockWidget)
         return self._curvesROIDockWidget
 
     @property
-    @deprecated
+    @deprecated(replacement="getMaskToolsDockWidget()", since_version="0.4.0")
     def maskToolsDockWidget(self):
         return self.getMaskToolsDockWidget()
 
@@ -405,12 +462,12 @@ class PlotWindow(PlotWidget):
             self._maskToolsDockWidget = MaskToolsDockWidget(
                 plot=self, name='Mask')
             self._maskToolsDockWidget.hide()
-            self._introduceNewDockWidget(self._maskToolsDockWidget)
+            self.addTabbedDockWidget(self._maskToolsDockWidget)
         return self._maskToolsDockWidget
 
     # getters for actions
     @property
-    @deprecated
+    @deprecated(replacement="getConsoleAction()", since_version="0.4.0")
     def consoleAction(self):
         return self.getConsoleAction()
 
@@ -434,21 +491,21 @@ class PlotWindow(PlotWidget):
         return self._consoleAction
 
     @property
-    @deprecated
+    @deprecated(replacement="getCrosshairAction()", since_version="0.4.0")
     def crosshairAction(self):
         return self.getCrosshairAction()
 
     def getCrosshairAction(self):
         """Action toggling crosshair cursor mode.
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         if self._crosshairAction is None:
-            self._crosshairAction = PlotActions.CrosshairAction(self, color='red')
+            self._crosshairAction = actions.control.CrosshairAction(self, color='red')
         return self._crosshairAction
 
     @property
-    @deprecated
+    @deprecated(replacement="getMaskAction()", since_version="0.4.0")
     def maskAction(self):
         return self.getMaskAction()
 
@@ -460,21 +517,22 @@ class PlotWindow(PlotWidget):
         return self.getMaskToolsDockWidget().toggleViewAction()
 
     @property
-    @deprecated
+    @deprecated(replacement="getPanWithArrowKeysAction()",
+                since_version="0.4.0")
     def panWithArrowKeysAction(self):
         return self.getPanWithArrowKeysAction()
 
     def getPanWithArrowKeysAction(self):
         """Action toggling pan with arrow keys.
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         if self._panWithArrowKeysAction is None:
-            self._panWithArrowKeysAction = PlotActions.PanWithArrowKeysAction(self)
+            self._panWithArrowKeysAction = actions.control.PanWithArrowKeysAction(self)
         return self._panWithArrowKeysAction
 
     @property
-    @deprecated
+    @deprecated(replacement="getRoiAction()", since_version="0.4.0")
     def roiAction(self):
         return self.getRoiAction()
 
@@ -488,63 +546,63 @@ class PlotWindow(PlotWidget):
     def getResetZoomAction(self):
         """Action resetting the zoom
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.resetZoomAction
 
     def getZoomInAction(self):
         """Action to zoom in
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.zoomInAction
 
     def getZoomOutAction(self):
         """Action to zoom out
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.zoomOutAction
 
     def getXAxisAutoScaleAction(self):
         """Action to toggle the X axis autoscale on zoom reset
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.xAxisAutoScaleAction
 
     def getYAxisAutoScaleAction(self):
         """Action to toggle the Y axis autoscale on zoom reset
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.yAxisAutoScaleAction
 
     def getXAxisLogarithmicAction(self):
         """Action to toggle logarithmic X axis
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.xAxisLogarithmicAction
 
     def getYAxisLogarithmicAction(self):
         """Action to toggle logarithmic Y axis
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.yAxisLogarithmicAction
 
     def getGridAction(self):
         """Action to toggle the grid visibility in the plot
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.gridAction
 
     def getCurveStyleAction(self):
         """Action to change curve line and markers styles
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.curveStyleAction
 
@@ -552,7 +610,7 @@ class PlotWindow(PlotWidget):
         """Action open a colormap dialog to change active image
         and default colormap.
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.colormapAction
 
@@ -568,7 +626,7 @@ class PlotWindow(PlotWidget):
         Use this to change the visibility of keepDataAspectRatioButton in the
         toolbar (See :meth:`QToolBar.addWidget` documentation).
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.keepDataAspectRatioButton
 
@@ -584,45 +642,59 @@ class PlotWindow(PlotWidget):
         Use this to change the visibility yAxisInvertedButton in the toolbar.
         (See :meth:`QToolBar.addWidget` documentation).
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.yAxisInvertedAction
 
     def getIntensityHistogramAction(self):
         """Action toggling the histogram intensity Plot widget
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self._intensityHistoAction
 
     def getCopyAction(self):
         """Action to copy plot snapshot to clipboard
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.copyAction
 
     def getSaveAction(self):
         """Action to save plot
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.saveAction
 
     def getPrintAction(self):
         """Action to print plot
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.printAction
 
     def getFitAction(self):
         """Action to fit selected curve
 
-        :rtype: PlotActions.PlotAction
+        :rtype: actions.PlotAction
         """
         return self.fitAction
 
+    def getMedianFilter1DAction(self):
+        """Action toggling the 1D median filter
+
+        :rtype: actions.PlotAction
+        """
+        return self._medianFilter1DAction
+
+    def getMedianFilter2DAction(self):
+        """Action toggling the 2D median filter
+
+        :rtype: actions.PlotAction
+        """
+        return self._medianFilter2DAction
+
 
 class Plot1D(PlotWindow):
     """PlotWindow with tools specific for curves.
@@ -630,10 +702,13 @@ class Plot1D(PlotWindow):
     This widgets provides the plot API of :class:`.PlotWidget`.
 
     :param parent: The parent of this widget
+    :param backend: The backend to use for the plot (default: matplotlib).
+                    See :class:`.PlotWidget` for the list of supported backend.
+    :type backend: str or :class:`BackendBase.BackendBase`
     """
 
-    def __init__(self, parent=None):
-        super(Plot1D, self).__init__(parent=parent, backend=None,
+    def __init__(self, parent=None, backend=None):
+        super(Plot1D, self).__init__(parent=parent, backend=backend,
                                      resetzoom=True, autoScale=True,
                                      logScale=True, grid=True,
                                      curveStyle=True, colormap=False,
@@ -643,26 +718,29 @@ class Plot1D(PlotWindow):
                                      roi=True, mask=False, fit=True)
         if parent is None:
             self.setWindowTitle('Plot1D')
-        self.setGraphXLabel('X')
-        self.setGraphYLabel('Y')
+        self.getXAxis().setLabel('X')
+        self.getYAxis().setLabel('Y')
 
 
 class Plot2D(PlotWindow):
     """PlotWindow with a toolbar specific for images.
 
-    This widgets provides the plot API of :class:`.PlotWidget`.
+    This widgets provides the plot API of :~:`.PlotWidget`.
 
     :param parent: The parent of this widget
+    :param backend: The backend to use for the plot (default: matplotlib).
+                    See :class:`.PlotWidget` for the list of supported backend.
+    :type backend: str or :class:`BackendBase.BackendBase`
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, backend=None):
         # List of information to display at the bottom of the plot
         posInfo = [
             ('X', lambda x, y: x),
             ('Y', lambda x, y: y),
             ('Data', self._getImageValue)]
 
-        super(Plot2D, self).__init__(parent=parent, backend=None,
+        super(Plot2D, self).__init__(parent=parent, backend=backend,
                                      resetzoom=True, autoScale=False,
                                      logScale=False, grid=False,
                                      curveStyle=False, colormap=True,
@@ -672,35 +750,60 @@ class Plot2D(PlotWindow):
                                      roi=False, mask=True)
         if parent is None:
             self.setWindowTitle('Plot2D')
-        self.setGraphXLabel('Columns')
-        self.setGraphYLabel('Rows')
+        self.getXAxis().setLabel('Columns')
+        self.getYAxis().setLabel('Rows')
 
         self.profile = ProfileToolBar(plot=self)
-
         self.addToolBar(self.profile)
 
+        self.getColorBarWidget().setVisible(True)
+
+        # Put colorbar action after colormap action
+        actions = self.toolBar().actions()
+        for index, action in enumerate(actions):
+            if action is self.getColormapAction():
+                break
+        self.toolBar().insertAction(
+            actions[index + 1],
+            self.getColorBarWidget().getToggleViewAction())
+
     def _getImageValue(self, x, y):
-        """Get value of top most image at position (x, y)
+        """Get status bar value of top most image at position (x, y)
 
         :param float x: X position in plot coordinates
         :param float y: Y position in plot coordinates
         :return: The value at that point or '-'
         """
         value = '-'
-        valueZ = - float('inf')
+        valueZ = -float('inf')
+        mask = 0
+        maskZ = -float('inf')
 
         for image in self.getAllImages():
-            data, params = image[0], image[4]
-            if params['z'] >= valueZ:  # This image is over the previous one
-                ox, oy = params['origin']
-                sx, sy = params['scale']
+            data = image.getData(copy=False)
+            isMask = isinstance(image, items.MaskImageData)
+            if isMask:
+                zIndex = maskZ
+            else:
+                zIndex = valueZ
+            if image.getZValue() >= zIndex:
+                # This image is over the previous one
+                ox, oy = image.getOrigin()
+                sx, sy = image.getScale()
                 row, col = (y - oy) / sy, (x - ox) / sx
                 if row >= 0 and col >= 0:
                     # Test positive before cast otherwise issue with int(-0.5) = 0
                     row, col = int(row), int(col)
                     if (row < data.shape[0] and col < data.shape[1]):
-                        value = data[row, col]
-                        valueZ = params['z']
+                        v, z = data[row, col], image.getZValue()
+                        if not isMask:
+                            value = v
+                            valueZ = z
+                        else:
+                            mask = v
+                            maskZ = z
+        if maskZ > valueZ and mask > 0:
+            return value, "Masked"
         return value
 
     def getProfileToolbar(self):
@@ -710,9 +813,13 @@ class Plot2D(PlotWindow):
         """
         return self.profile
 
+    @deprecated(replacement="getProfilePlot", since_version="0.5.0")
     def getProfileWindow(self):
-        """Plot window used to display profile curve.
+        return self.getProfilePlot()
+
+    def getProfilePlot(self):
+        """Return plot window used to display profile curve.
 
         :return: :class:`Plot1D`
         """
-        return self.profile.profileWindow
+        return self.profile.getProfilePlot()
