@@ -69,25 +69,31 @@ Example::
 
 __authors__ = ["P. Knobel", "H. Payno"]
 __license__ = "MIT"
-__date__ = "20/01/2017"
+__date__ = "11/09/2017"
 
 import numpy
 
-try:
-    import h5py
-except ImportError:
-    h5py = None
-
 from silx.gui import qt
 from .. import icons
-from . import PlotWindow
-from . import PlotActions
+from . import items, PlotWindow, actions
+from .Colormap import Colormap
 from .Colors import cursorColorForColormap
 from .PlotTools import LimitsToolBar
 from .Profile import Profile3DToolBar
 from ..widgets.FrameBrowser import HorizontalSliderWithBrowser
 
 from silx.utils.array_like import DatasetView, ListOfImages
+from silx.math import calibration
+from silx.utils.deprecation import deprecated_warning
+
+try:
+    import h5py
+except ImportError:
+    def is_dataset(obj):
+        return False
+    h5py = None
+else:
+    from silx.io.utils import is_dataset
 
 
 class StackView(qt.QMainWindow):
@@ -99,8 +105,8 @@ class StackView(qt.QMainWindow):
     and display the result as a slice.
 
     :param QWidget parent: the Qt parent, or None
-    :param backend: The backend to use for the plot.
-                    The default is to use matplotlib.
+    :param backend: The backend to use for the plot (default: matplotlib).
+                    See :class:`.PlotWidget` for the list of supported backend.
     :type backend: str or :class:`BackendBase.BackendBase`
     :param bool resetzoom: Toggle visibility of reset zoom action.
     :param bool autoScale: Toggle visibility of axes autoscale actions.
@@ -178,6 +184,13 @@ class StackView(qt.QMainWindow):
         """These labels are displayed on the X and Y axes.
         :meth:`setLabels` updates this attribute."""
 
+        self._first_stack_dimension = 0
+        """Used for dimension labels and combobox"""
+
+        self._titleCallback = self._defaultTitleCallback
+        """Function returning the plot title based on the frame index.
+        It can be set to a custom function using :meth:`setTitleCallback`"""
+
         central_widget = qt.QWidget(self)
 
         self._plot = PlotWindow(parent=central_widget, backend=backend,
@@ -192,41 +205,49 @@ class StackView(qt.QMainWindow):
         self.sigActiveImageChanged = self._plot.sigActiveImageChanged
         self.sigPlotSignal = self._plot.sigPlotSignal
 
+        self._addColorBarAction()
+
         self._plot.profile = Profile3DToolBar(parent=self._plot,
-                                              plot=self)
+                                              stackview=self)
         self._plot.addToolBar(self._plot.profile)
-        self._plot.setGraphXLabel('Columns')
-        self._plot.setGraphYLabel('Rows')
+        self._plot.getXAxis().setLabel('Columns')
+        self._plot.getYAxis().setLabel('Rows')
         self._plot.sigPlotSignal.connect(self._plotCallback)
 
+        self.__planeSelection = PlanesWidget(self._plot)
+        self.__planeSelection.sigPlaneSelectionChanged.connect(self.__setPerspective)
+
+        self._browser_label = qt.QLabel("Image index (Dim0):")
+
         self._browser = HorizontalSliderWithBrowser(central_widget)
         self._browser.valueChanged[int].connect(self.__updateFrameNumber)
         self._browser.setEnabled(False)
 
-        self.__planeSelection = PlanesWidget(self._plot)
-        self.__planeSelection.sigPlaneSelectionChanged.connect(self.__setPerspective)
-
         layout = qt.QGridLayout()
         layout.setContentsMargins(0, 0, 0, 0)
-        layout.addWidget(self._plot, 0, 0, 1, 2)
+        layout.addWidget(self._plot, 0, 0, 1, 3)
         layout.addWidget(self.__planeSelection, 1, 0)
-        layout.addWidget(self._browser, 1, 1)
+        layout.addWidget(self._browser_label, 1, 1)
+        layout.addWidget(self._browser, 1, 2)
 
         central_widget.setLayout(layout)
         self.setCentralWidget(central_widget)
 
-        # clear profile lines when the profile changes (plane browsed changed)
+        # clear profile lines when the perspective changes (plane browsed changed)
         self.__planeSelection.sigPlaneSelectionChanged.connect(
-            self._plot.profile.hideProfileWindow)
+            self._plot.profile.getProfilePlot().clear)
+        self.__planeSelection.sigPlaneSelectionChanged.connect(
+            self._plot.profile.clearProfile)
 
-    def setOptionVisible(self, isVisible):
-        """
-        Set the visibility of the browsing options.
-
-        :param bool isVisible: True to have the options visible, else False
-        """
-        self._browser.setVisible(isVisible)
-        self.__planeSelection.setVisible(isVisible)
+    def _addColorBarAction(self):
+        self._plot.getColorBarWidget().setVisible(True)
+        actions = self._plot.toolBar().actions()
+        for index, action in enumerate(actions):
+            if action is self._plot.getColormapAction():
+                break
+        self._plot.toolBar().insertAction(
+            actions[index + 1],
+            self._plot.getColorBarWidget().getToggleViewAction())
 
     def _plotCallback(self, eventDict):
         """Callback for plot events.
@@ -234,14 +255,14 @@ class StackView(qt.QMainWindow):
         Emit :attr:`valueChanged` signal, with (x, y, value) tuple of the
         cursor location in the plot."""
         if eventDict['event'] == 'mouseMoved':
-            activeImage = self.getActiveImage()
+            activeImage = self._plot.getActiveImage()
             if activeImage is not None:
-                data = activeImage[0]
+                data = activeImage.getData()
                 height, width = data.shape
 
                 # Get corresponding coordinate in image
-                origin = activeImage[4]['origin']
-                scale = activeImage[4]['scale']
+                origin = activeImage.getOrigin()
+                scale = activeImage.getScale()
                 x = int((eventDict['x'] - origin[0]) / scale[0])
                 y = int((eventDict['y'] - origin[1]) / scale[1])
 
@@ -253,22 +274,26 @@ class StackView(qt.QMainWindow):
                                            None)
 
     def __setPerspective(self, perspective):
-        """Function called when the browsed/orthogonal dimension changes
+        """Function called when the browsed/orthogonal dimension changes.
+        Updates :attr:`_perspective`, transposes data, updates the plot,
+        emits :attr:`sigPlaneSelectionChanged` and :attr:`sigStackChanged`.
 
-        :param perspective: the new browsed dimension
+        :param int perspective: the new browsed dimension
         """
         if perspective == self._perspective:
             return
         else:
             if perspective > 2 or perspective < 0:
                 raise ValueError(
-                        "Perspective must be 0, 1 or 2, not %s" % perspective)
+                    "Perspective must be 0, 1 or 2, not %s" % perspective)
 
             self._perspective = perspective
             self.__createTransposedView()
             self.__updateFrameNumber(self._browser.value())
             self._plot.resetZoom()
             self.__updatePlotLabels()
+            self._browser_label.setText("Image index (Dim%d):" %
+                                        (self._first_stack_dimension + perspective))
 
             self.sigPlaneSelectionChanged.emit(perspective)
             self.sigStackChanged.emit(self._stack.size if
@@ -293,8 +318,7 @@ class StackView(qt.QMainWindow):
         if isinstance(self._stack, numpy.ndarray):
             self.__transposed_view = self._stack
 
-        elif h5py is not None and isinstance(self._stack, h5py.Dataset) or \
-                isinstance(self._stack, DatasetView):
+        elif is_dataset(self._stack) or isinstance(self._stack, DatasetView):
             self.__transposed_view = DatasetView(self._stack)
 
         elif isinstance(self._stack, ListOfImages):
@@ -309,13 +333,6 @@ class StackView(qt.QMainWindow):
         self._browser.setRange(0, self.__transposed_view.shape[0] - 1)
         self._browser.setValue(0)
 
-    def setFrameNumber(self, number):
-        """Set the frame selection to a specific value\
-
-        :param int number: Number of the frame
-        """
-        self._browser.setValue(number)
-
     def __updateFrameNumber(self, index):
         """Update the current image.
 
@@ -323,11 +340,76 @@ class StackView(qt.QMainWindow):
         """
         assert self.__transposed_view is not None
         self._plot.addImage(self.__transposed_view[index, :, :],
+                            origin=self._getImageOrigin(),
+                            scale=self._getImageScale(),
                             legend=self.__imageLegend,
-                            resetzoom=False)
+                            resetzoom=False, replace=False)
+        self._updateTitle()
 
-    # public API
-    def setStack(self, stack, perspective=0, reset=True):
+    def _set3DScaleAndOrigin(self, calibrations):
+        """Set scale and origin for all 3 axes, to be used when plotting
+        an image.
+
+        See setStack for parameter documentation
+        """
+        if calibrations is None:
+            self.calibrations3D = (calibration.NoCalibration(),
+                                   calibration.NoCalibration(),
+                                   calibration.NoCalibration())
+        else:
+            self.calibrations3D = []
+            for calib in calibrations:
+                if hasattr(calib, "__len__") and len(calib) == 2:
+                    calib = calibration.LinearCalibration(calib[0], calib[1])
+                elif calib is None:
+                    calib = calibration.NoCalibration()
+                elif not isinstance(calib, calibration.AbstractCalibration):
+                    raise TypeError("calibration must be a 2-tuple, None or" +
+                                    " an instance of an AbstractCalibration " +
+                                    "subclass")
+                self.calibrations3D.append(calib)
+
+    def _getXYZCalibs(self):
+        xy_dims = [0, 1, 2]
+        xy_dims.remove(self._perspective)
+
+        xcalib = self.calibrations3D[max(xy_dims)]
+        ycalib = self.calibrations3D[min(xy_dims)]
+        zcalib = self.calibrations3D[self._perspective]
+
+        return xcalib, ycalib, zcalib
+
+    def _getImageScale(self):
+        """
+        :return: 2-tuple (XScale, YScale) for current image view
+        """
+        xcalib, ycalib, _zcalib = self._getXYZCalibs()
+        return xcalib.get_slope(), ycalib.get_slope()
+
+    def _getImageOrigin(self):
+        """
+        :return: 2-tuple (XOrigin, YOrigin) for current image view
+        """
+        xcalib, ycalib, _zcalib = self._getXYZCalibs()
+        return xcalib(0), ycalib(0)
+
+    def _getImageZ(self, index):
+        """
+        :param idx: 0-based image index in the stack
+        :return: calibrated Z value corresponding to the image idx
+        """
+        _xcalib, _ycalib, zcalib = self._getXYZCalibs()
+        return zcalib(index)
+
+    def _updateTitle(self):
+        frame_idx = self._browser.value()
+        self._plot.setGraphTitle(self._titleCallback(frame_idx))
+
+    def _defaultTitleCallback(self, index):
+        return "Image z=%g" % self._getImageZ(index)
+
+    # public API, stack specific methods
+    def setStack(self, stack, perspective=0, reset=True, calibrations=None):
         """Set the 3D stack.
 
         The perspective parameter is used to define which dimension of the 3D
@@ -342,15 +424,21 @@ class StackView(qt.QMainWindow):
             By default, the dimension for the image index is the first
             dimension of the 3D stack (``perspective=0``).
         :param bool reset: Whether to reset zoom or not.
+        :param calibrations: Sequence of 3 calibration objects for each axis.
+            These objects can be a subclass of :class:`AbstractCalibration`,
+            or 2-tuples *(a, b)* where *a* is the y-intercept and *b* is the
+            slope of a linear calibration (:math:`x \mapsto a + b x`)
         """
         if stack is None:
             self.clear()
             self.sigStackChanged.emit(0)
             return
 
+        self._set3DScaleAndOrigin(calibrations)
+
         # stack as list of 2D arrays: must be converted into an array_like
         if not isinstance(stack, numpy.ndarray):
-            if h5py is None or not isinstance(stack, h5py.Dataset):
+            if not is_dataset(stack):
                 try:
                     assert hasattr(stack, "__len__")
                     for img in stack:
@@ -367,6 +455,9 @@ class StackView(qt.QMainWindow):
         self._stack = stack
         self.__createTransposedView()
 
+        if perspective != self._perspective:
+            self.__setPerspective(perspective)
+
         # This call to setColormap redefines the meaning of autoscale
         # for 3D volume: take global min/max rather than frame min/max
         if self.__autoscaleCmap:
@@ -376,8 +467,11 @@ class StackView(qt.QMainWindow):
         self._plot.addImage(self.__transposed_view[0, :, :],
                             legend=self.__imageLegend,
                             colormap=self.getColormap(),
+                            origin=self._getImageOrigin(),
+                            scale=self._getImageScale(),
                             resetzoom=False)
         self._plot.setActiveImage(self.__imageLegend)
+        self._plot.setGraphTitle("Image z=%g" % self._getImageZ(0))
         self.__updatePlotLabels()
 
         if reset:
@@ -387,9 +481,12 @@ class StackView(qt.QMainWindow):
         self._browser.setEnabled(True)
 
         if perspective != self._perspective:
-            self.__setPerspective(perspective)
+            self.__planeSelection.setPerspective(perspective)
+            # this causes self.__setPerspective to be called, which emits
+            # sigStackChanged and sigPlaneSelectionChanged
 
-        self.sigStackChanged.emit(stack.size)
+        else:
+            self.sigStackChanged.emit(stack.size)
 
     def getStack(self, copy=True, returnNumpyArray=False):
         """Get the original stack, as a 3D array or dataset.
@@ -407,9 +504,26 @@ class StackView(qt.QMainWindow):
         :return: 3D stack and parameters.
         :rtype: (numpy.ndarray, dict)
         """
-        if self.getActiveImage() is None:
+        image = self._plot.getActiveImage()
+        if image is None:
             return None
-        _img, _legend, _info, _pixmap, params = self.getActiveImage()
+
+        if isinstance(image, items.ColormapMixIn):
+            colormap = image.getColormap()
+        else:
+            colormap = None
+
+        params = {
+            'info': image.getInfo(),
+            'origin': image.getOrigin(),
+            'scale': image.getScale(),
+            'z': image.getZValue(),
+            'selectable': image.isSelectable(),
+            'draggable': image.isDraggable(),
+            'colormap': colormap,
+            'xlabel': image.getXLabel(),
+            'ylabel': image.getYLabel(),
+        }
         if returnNumpyArray or copy:
             return numpy.array(self._stack, copy=copy), params
 
@@ -441,28 +555,81 @@ class StackView(qt.QMainWindow):
         :return: 3D stack and parameters.
         :rtype: (numpy.ndarray, dict)
         """
-        if self.getActiveImage() is None:
+        image = self._plot.getActiveImage()
+        if image is None:
             return None
-        _img, _legend, _info, _pixmap, params = self.getActiveImage()
+
+        if isinstance(image, items.ColormapMixIn):
+            colormap = image.getColormap()
+        else:
+            colormap = None
+
+        params = {
+            'info': image.getInfo(),
+            'origin': image.getOrigin(),
+            'scale': image.getScale(),
+            'z': image.getZValue(),
+            'selectable': image.isSelectable(),
+            'draggable': image.isDraggable(),
+            'colormap': colormap,
+            'xlabel': image.getXLabel(),
+            'ylabel': image.getYLabel(),
+        }
         if returnNumpyArray or copy:
             return numpy.array(self.__transposed_view, copy=copy), params
         return self.__transposed_view, params
 
-    def getActiveImage(self, just_legend=False):
-        """Returns the currently active image.
+    def setFrameNumber(self, number):
+        """Set the frame selection to a specific value
 
-        It returns None in case of not having an active image.
-
-        Default output has the form: [data, legend, info, pixmap, params]
-        where params is a dictionary containing image parameters.
-
-        :param bool just_legend: True to get the legend of the image,
-            False (the default) to get the image data and info.
-            Note: :class:`StackView` uses the same legend for all frames.
-        :return: legend or [data, legend, info, pixmap, params]
-        :rtype: str or list
+        :param int number: Number of the frame
         """
-        return self._plot.getActiveImage(just_legend=just_legend)
+        self._browser.setValue(number)
+
+    def setFirstStackDimension(self, first_stack_dimension):
+        """When viewing the last 3 dimensions of an n-D array (n>3), you can
+        use this method to change the text in the combobox.
+
+        For instance, for a 7-D array, first stack dim is 4, so the default
+        "Dim1-Dim2" text should be replaced with "Dim5-Dim6" (dimensions
+        numbers are 0-based).
+
+        :param int first_stack_dim: First stack dimension (n-3) when viewing the
+            last 3 dimensions of an n-D array.
+        """
+        old_state = self.__planeSelection.blockSignals(True)
+        self.__planeSelection.setFirstStackDimension(first_stack_dimension)
+        self.__planeSelection.blockSignals(old_state)
+        self._first_stack_dimension = first_stack_dimension
+        self._browser_label.setText("Image index (Dim%d):" % first_stack_dimension)
+
+    def setTitleCallback(self, callback):
+        """Set a user defined function to generate the plot title based on the
+        image/frame index.
+
+        The callback function must accept an integer as a its first positional
+        parameter and must not require any other mandatory parameter.
+        It must return a string.
+
+        To switch back the default behavior, you can pass ``None``::
+
+            mystackview.setTitleCallback(None)
+
+        To have no title, pass a function that returns an empty string::
+
+            mystackview.setTitleCallback(lambda idx: "")
+
+        :param callback: Callback function generating the stack title based
+            on the frame number.
+        """
+
+        if callback is None:
+            self._titleCallback = self._defaultTitleCallback
+        elif callable(callback):
+            self._titleCallback = callback
+        else:
+            raise TypeError("Provided callback is not callable")
+        self._updateTitle()
 
     def clear(self):
         """Clear the widget:
@@ -476,22 +643,6 @@ class StackView(qt.QMainWindow):
         self._browser.setEnabled(False)
         self._plot.clear()
 
-    def resetZoom(self):
-        """Reset the plot limits to the bounds of the data and redraw the plot.
-        """
-        self._plot.resetZoom()
-
-    def getGraphTitle(self):
-        """Return the plot main title as a str."""
-        return self._plot.getGraphTitle()
-
-    def setGraphTitle(self, title=""):
-        """Set the plot main title.
-
-        :param str title: Main title of the plot (default: '')
-        """
-        return self._plot.setGraphTitle(title)
-
     def setLabels(self, labels=None):
         """Set the labels to be displayed on the plot axes.
 
@@ -504,60 +655,28 @@ class StackView(qt.QMainWindow):
         :param list(str) labels: 3 labels corresponding to the 3 dimensions
              of the data volumes.
         """
+
+        default_labels = ["Dimension %d" % self._first_stack_dimension,
+                          "Dimension %d" % (self._first_stack_dimension + 1),
+                          "Dimension %d" % (self._first_stack_dimension + 2)]
         if labels is None:
-            labels = ["Dimension 0", "Dimension 1", "Dimension 2"]
-        self.__dimensionsLabels = labels
+            new_labels = default_labels
+        else:
+            # filter-out None
+            new_labels = []
+            for i, label in enumerate(labels):
+                new_labels.append(label or default_labels[i])
+
+        self.__dimensionsLabels = new_labels
         self.__updatePlotLabels()
 
-    def getGraphXLabel(self):
-        """Return the current horizontal axis label as a str."""
-        return self._plot.getGraphXLabel()
+    def getLabels(self):
+        """Return dimension labels displayed on the plot axes
 
-    def setGraphXLabel(self, label=None):
-        """Set the plot horizontal axis label.
-
-        :param str label: The horizontal axis label
+        :return: List of three strings corresponding to the 3 dimensions
+            of the stack: (name_dim0, name_dim1, name_dim2)
         """
-        if label is None:
-            label = self.__dimensionsLabels[1 if self._perspective == 2 else 2]
-        self._plot.setGraphXLabel(label)
-
-    def getGraphYLabel(self, axis='left'):
-        """Return the current vertical axis label as a str.
-
-        :param str axis: The Y axis for which to get the label (left or right)
-        """
-        return self._plot.getGraphYLabel(axis)
-
-    def setGraphYLabel(self, label=None, axis='left'):
-        """Set the vertical axis label on the plot.
-
-        :param str label: The Y axis label
-        :param str axis: The Y axis for which to set the label (left or right)
-        """
-        if label is None:
-            label = self.__dimensionsLabels[1 if self._perspective == 0 else 0]
-        self._plot.setGraphYLabel(label, axis)
-
-    def setYAxisInverted(self, flag=True):
-        """Set the Y axis orientation.
-
-        :param bool flag: True for Y axis going from top to bottom,
-                          False for Y axis going from bottom to top
-        """
-        self._plot.setYAxisInverted(flag)
-
-    def isYAxisInverted(self):
-        """Return True if Y axis goes from top to bottom, False otherwise."""
-        return self._backend.isYAxisInverted()
-
-    def getSupportedColormaps(self):
-        """Get the supported colormap names as a tuple of str.
-
-        The list should at least contain and start by:
-        ('gray', 'reversed gray', 'temperature', 'red', 'green', 'blue')
-        """
-        return self._plot.getSupportedColormaps()
+        return self.__dimensionsLabels
 
     def getColormap(self):
         """Get the current colormap description.
@@ -594,12 +713,12 @@ class StackView(qt.QMainWindow):
 
         :param colormap: Name of the colormap in
             'gray', 'reversed gray', 'temperature', 'red', 'green', 'blue'.
-            Or the description of the colormap as a dict.
+            Or a :class`.Colormap` object.
         :type colormap: dict or str.
         :param str normalization: Colormap mapping: 'linear' or 'log'.
         :param bool autoscale: Whether to use autoscale or [vmin, vmax] range.
-            Default value of autoscale is True if data is a numpy array,
-            False if data is a h5py dataset.
+            Default value of autoscale is False. This option is not compatible
+            with h5py datasets.
         :param float vmin: The minimum value of the range to use if
                            'autoscale' is False.
         :param float vmax: The maximum value of the range to use if
@@ -607,87 +726,85 @@ class StackView(qt.QMainWindow):
         :param numpy.ndarray colors: Only used if name is None.
             Custom colormap colors as Nx3 or Nx4 RGB or RGBA arrays
         """
-        cmapDict = self.getColormap()
-
-        if isinstance(colormap, dict):
+        # if is a colormap object or a dictionary
+        if isinstance(colormap, Colormap) or isinstance(colormap, dict):
             # Support colormap parameter as a dict
-            errmsg = "If colormap is provided as a dict, all other parameters"
+            errmsg = "If colormap is provided as a Colormap object, all other parameters"
             errmsg += " must not be specified when calling setColormap"
             assert normalization is None, errmsg
             assert autoscale is None, errmsg
             assert vmin is None, errmsg
             assert vmax is None, errmsg
             assert colors is None, errmsg
-            cmapDict.update(colormap)
 
+            if isinstance(colormap, dict):
+                reason = 'colormap parameter should now be an object'
+                replacement = 'Colormap()'
+                since_version = '0.6'
+                deprecated_warning(type_='function',
+                                   name='setColormap',
+                                   reason=reason,
+                                   replacement=replacement,
+                                   since_version=since_version)
+                _colormap = Colormap._fromDict(colormap)
+            else:
+                _colormap = colormap
         else:
-            if colormap is not None:
-                cmapDict['name'] = colormap
-            if normalization is not None:
-                cmapDict['normalization'] = normalization
-            if colors is not None:
-                cmapDict['colors'] = colors
-
-            # Default meaning of autoscale is to reset min and max
-            # each time a new image is added to the plot.
-            # We want to use min and max of global volume,
-            # and not change them when browsing slides
-            cmapDict['autoscale'] = False
+            norm = normalization if normalization is not None else 'linear'
+            name = colormap if colormap is not None else 'gray'
+            _colormap = Colormap(name=name,
+                                 normalization=norm,
+                                 vmin=vmin,
+                                 vmax=vmax,
+                                 colors=colors)
 
+            # Patch: since we don't apply this colormap to a single 2D data but
+            # a 2D stack we have to deal manually with vmin, vmax
             if autoscale is None:
                 # set default
                 autoscale = False
-                # TODO: assess cost of computing min/max for large 3D array
-                # if isinstance(self._stack, numpy.ndarray):
-                #     autoscale = True
-                # else:                    # h5py.Dataset
-                #     autoscale = False
-            elif autoscale and isinstance(self._stack, h5py.Dataset):
+            elif autoscale and is_dataset(self._stack):
                 # h5py dataset has no min()/max() methods
                 raise RuntimeError(
-                        "Cannot auto-scale colormap for a h5py dataset")
+                    "Cannot auto-scale colormap for a h5py dataset")
             else:
                 autoscale = autoscale
             self.__autoscaleCmap = autoscale
-            if autoscale and (self._stack is not None):
-                cmapDict['vmin'] = self._stack.min()
-                cmapDict['vmax'] = self._stack.max()
-            else:
-                if vmin is not None:
-                    cmapDict['vmin'] = vmin
-                if vmax is not None:
-                    cmapDict['vmax'] = vmax
 
-        cursorColor = cursorColorForColormap(cmapDict['name'])
+            if autoscale and (self._stack is not None):
+                _vmin, _vmax = _colormap.getColormapRange(data=self._stack)
+                _colormap.setVRange(vmin=_vmin, vmax=_vmax)
+            else:
+                if vmin is None and self._stack is not None:
+                    _colormap.setVMin(self._stack.min())
+                else:
+                    _colormap.setVMin(vmin)
+                if vmax is None and self._stack is not None:
+                    _colormap.setVMax(self._stack.max())
+                else:
+                    _colormap.setVMax(vmax)
+
+        cursorColor = cursorColorForColormap(_colormap.getName())
         self._plot.setInteractiveMode('zoom', color=cursorColor)
 
-        self._plot.setDefaultColormap(cmapDict)
+        self._plot.setDefaultColormap(_colormap)
 
-        # Refresh image with new colormap
+        # Update active image colormap
         activeImage = self._plot.getActiveImage()
-        if activeImage is not None:
-            data, legend, info, _pixmap = activeImage[0:4]
-            self._plot.addImage(data, legend=legend, info=info,
-                                colormap=self.getColormap(),
-                                resetzoom=False)
+        if isinstance(activeImage, items.ColormapMixIn):
+            activeImage.setColormap(self.getColormap())
 
-    def isKeepDataAspectRatio(self):
-        """Returns whether the plot is keeping data aspect ratio or not."""
-        return self._plot.isKeepDataAspectRatio()
+    def getPlot(self):
+        """Return the :class:`PlotWidget`.
 
-    def setKeepDataAspectRatio(self, flag=True):
-        """Set whether the plot keeps data aspect ratio or not.
+        This gives access to advanced plot configuration options.
+        Be warned that modifying the plot can cause issues, and some changes
+        you make to the plot could be overwritten by the :class:`StackView`
+        widget's internal methods and callbacks.
 
-        :param bool flag: True to respect data aspect ratio
+        :return: instance of :class:`PlotWidget` used in widget
         """
-        self._plot.setKeepDataAspectRatio(flag)
-
-    def getProfileToolbar(self):
-        """Profile tools attached to this plot
-
-        See :class:`silx.gui.plot.Profile.Profile3DToolBar`
-        """
-        return self._plot.profile
+        return self._plot
 
     def getProfileWindow1D(self):
         """Plot window used to display 1D profile curve.
@@ -703,7 +820,158 @@ class StackView(qt.QMainWindow):
         """
         return self._plot.profile.getProfileWindow2D()
 
+    def setOptionVisible(self, isVisible):
+        """
+        Set the visibility of the browsing options.
+
+        :param bool isVisible: True to have the options visible, else False
+        """
+        self._browser.setVisible(isVisible)
+        self.__planeSelection.setVisible(isVisible)
+
+    # proxies to PlotWidget or PlotWindow methods
+    def getProfileToolbar(self):
+        """Profile tools attached to this plot
+
+        See :class:`silx.gui.plot.Profile.Profile3DToolBar`
+        """
+        return self._plot.profile
+
+    def getGraphTitle(self):
+        """Return the plot main title as a str.
+        """
+        return self._plot.getGraphTitle()
+
+    def setGraphTitle(self, title=""):
+        """Set the plot main title.
+
+        :param str title: Main title of the plot (default: '')
+        """
+        return self._plot.setGraphTitle(title)
+
+    def getGraphXLabel(self):
+        """Return the current horizontal axis label as a str.
+        """
+        return self._plot.getXAxis().getLabel()
+
+    def setGraphXLabel(self, label=None):
+        """Set the plot horizontal axis label.
+
+        :param str label: The horizontal axis label
+        """
+        if label is None:
+            label = self.__dimensionsLabels[1 if self._perspective == 2 else 2]
+        self._plot.getXAxis().setLabel(label)
+
+    def getGraphYLabel(self, axis='left'):
+        """Return the current vertical axis label as a str.
+
+        :param str axis: The Y axis for which to get the label (left or right)
+        """
+        return self._plot.getYAxis().getLabel(axis)
+
+    def setGraphYLabel(self, label=None, axis='left'):
+        """Set the vertical axis label on the plot.
+
+        :param str label: The Y axis label
+        :param str axis: The Y axis for which to set the label (left or right)
+        """
+        if label is None:
+            label = self.__dimensionsLabels[1 if self._perspective == 0 else 0]
+        self._plot.getYAxis(axis=axis).setLabel(label)
+
+    def resetZoom(self):
+        """Reset the plot limits to the bounds of the data and redraw the plot.
+
+        This method is a simple proxy to the legacy :class:`PlotWidget` method
+        of the same name. Using the object oriented approach is now
+        preferred::
+
+            stackview.getPlot().resetZoom()
+        """
+        self._plot.resetZoom()
+
+    def setYAxisInverted(self, flag=True):
+        """Set the Y axis orientation.
+
+        This method is a simple proxy to the legacy :class:`PlotWidget` method
+        of the same name. Using the object oriented approach is now
+        preferred::
+
+            stackview.getPlot().setYAxisInverted(flag)
+
+        :param bool flag: True for Y axis going from top to bottom,
+                          False for Y axis going from bottom to top
+        """
+        self._plot.setYAxisInverted(flag)
+
+    def isYAxisInverted(self):
+        """Return True if Y axis goes from top to bottom, False otherwise.
+
+        This method is a simple proxy to the legacy :class:`PlotWidget` method
+        of the same name. Using the object oriented approach is now
+        preferred::
+
+            stackview.getPlot().isYAxisInverted()"""
+        return self._plot.isYAxisInverted()
+
+    def getSupportedColormaps(self):
+        """Get the supported colormap names as a tuple of str.
+
+        The list should at least contain and start by:
+        ('gray', 'reversed gray', 'temperature', 'red', 'green', 'blue')
+
+        This method is a simple proxy to the legacy :class:`PlotWidget` method
+        of the same name. Using the object oriented approach is now
+        preferred::
+
+            stackview.getPlot().getSupportedColormaps()
+        """
+        return self._plot.getSupportedColormaps()
+
+    def isKeepDataAspectRatio(self):
+        """Returns whether the plot is keeping data aspect ratio or not.
+
+        This method is a simple proxy to the legacy :class:`PlotWidget` method
+        of the same name. Using the object oriented approach is now
+        preferred::
+
+            stackview.getPlot().isKeepDataAspectRatio()"""
+        return self._plot.isKeepDataAspectRatio()
+
+    def setKeepDataAspectRatio(self, flag=True):
+        """Set whether the plot keeps data aspect ratio or not.
+
+        This method is a simple proxy to the legacy :class:`PlotWidget` method
+        of the same name. Using the object oriented approach is now
+        preferred::
+
+            stackview.getPlot().setKeepDataAspectRatio(flag)
+
+        :param bool flag: True to respect data aspect ratio
+        """
+        self._plot.setKeepDataAspectRatio(flag)
+
     # kind of private methods, but needed by Profile
+    def getActiveImage(self, just_legend=False):
+        """Returns the currently active image object.
+
+        It returns None in case of not having an active image.
+
+        This method is a simple proxy to the legacy :class:`PlotWidget` method
+        of the same name. Using the object oriented approach is now
+        preferred::
+
+            stackview.getPlot().getActiveImage()
+
+        :param bool just_legend: True to get the legend of the image,
+            False (the default) to get the image data and info.
+            Note: :class:`StackView` uses the same legend for all frames.
+        :return: legend or image object
+        :rtype: str or list or None
+        """
+        return self._plot.getActiveImage(just_legend=just_legend)
+
     def remove(self, legend=None,
                kind=('curve', 'image', 'item', 'marker')):
         """See :meth:`Plot.Plot.remove`"""
@@ -746,16 +1014,10 @@ class PlanesWidget(qt.QWidget):
         # 1
         # | 0
         # |/__2
-
         self.qcbAxisSelection = qt.QComboBox(self)
-        self.qcbAxisSelection.addItem(icons.getQIcon("cube-front"),
-                                      'Dim1-Dim2')
-        self.qcbAxisSelection.addItem(icons.getQIcon("cube-bottom"),
-                                      'Dim0-Dim2')
-        self.qcbAxisSelection.addItem(icons.getQIcon("cube-left"),
-                                      'Dim0-Dim1')
+        self._setCBChoices(first_stack_dimension=0)
         self.qcbAxisSelection.currentIndexChanged[int].connect(
-                self.__planeSelectionChanged)
+            self.__planeSelectionChanged)
 
         layout0.addWidget(self.qcbAxisSelection)
 
@@ -771,6 +1033,44 @@ class PlanesWidget(qt.QWidget):
         """
         self.sigPlaneSelectionChanged.emit(idx)
 
+    def _setCBChoices(self, first_stack_dimension):
+        self.qcbAxisSelection.clear()
+
+        dim1dim2 = 'Dim%d-Dim%d' % (first_stack_dimension + 1,
+                                    first_stack_dimension + 2)
+        dim0dim2 = 'Dim%d-Dim%d' % (first_stack_dimension,
+                                    first_stack_dimension + 2)
+        dim0dim1 = 'Dim%d-Dim%d' % (first_stack_dimension,
+                                    first_stack_dimension + 1)
+
+        self.qcbAxisSelection.addItem(icons.getQIcon("cube-front"), dim1dim2)
+        self.qcbAxisSelection.addItem(icons.getQIcon("cube-bottom"), dim0dim2)
+        self.qcbAxisSelection.addItem(icons.getQIcon("cube-left"), dim0dim1)
+
+    def setFirstStackDimension(self, first_stack_dim):
+        """When viewing the last 3 dimensions of an n-D array (n>3), you can
+        use this method to change the text in the combobox.
+
+        For instance, for a 7-D array, first stack dim is 4, so the default
+        "Dim1-Dim2" text should be replaced with "Dim5-Dim6" (dimensions
+        numbers are 0-based).
+
+        :param int first_stack_dim: First stack dimension (n-3) when viewing the
+            last 3 dimensions of an n-D array.
+        """
+        self._setCBChoices(first_stack_dim)
+
+    def setPerspective(self, perspective):
+        """Update the combobox selection.
+
+          - slice plane Dim1-Dim2: perspective 0
+          - slice plane Dim0-Dim2: perspective 1
+          - slice plane Dim0-Dim1: perspective 2
+
+        :param perspective: Orthogonal dimension number (0, 1, or 2)
+        """
+        self.qcbAxisSelection.setCurrentIndex(perspective)
+
 
 class StackViewMainWindow(StackView):
     """This class is a :class:`StackView` with a menu, an additional toolbar
@@ -802,11 +1102,12 @@ class StackViewMainWindow(StackView):
         menu.addSeparator()
         menu.addAction(self._plot.resetZoomAction)
         menu.addAction(self._plot.colormapAction)
-        menu.addAction(PlotActions.KeepAspectRatioAction(self._plot, self))
-        menu.addAction(PlotActions.YAxisInvertedAction(self._plot, self))
+        menu.addAction(self._plot.getColorBarWidget().getToggleViewAction())
+
+        menu.addAction(actions.control.KeepAspectRatioAction(self._plot, self))
+        menu.addAction(actions.control.YAxisInvertedAction(self._plot, self))
 
         menu = self.menuBar().addMenu('Profile')
-        menu.addAction(self._plot.profile.browseAction)
         menu.addAction(self._plot.profile.hLineAction)
         menu.addAction(self._plot.profile.vLineAction)
         menu.addAction(self._plot.profile.lineAction)
