@@ -40,8 +40,8 @@ import numpy
 
 from silx.gui.plot.Colors import rgba
 
-from .. import glutils
-from ..glutils import gl
+from ... import _glutils
+from ..._glutils import gl
 
 from . import event
 from . import core
@@ -242,7 +242,7 @@ class Geometry(core.Elem):
                     assert len(array) in (1, 2, 3, 4)
                     gl.glDisableVertexAttribArray(attribute)
                     _glVertexAttribFunc = getattr(
-                        glutils.gl, 'glVertexAttrib{}f'.format(len(array)))
+                        _glutils.gl, 'glVertexAttrib{}f'.format(len(array)))
                     _glVertexAttribFunc(attribute, *array)
                 else:
                     # TODO As is this is a never event, remove?
@@ -250,7 +250,7 @@ class Geometry(core.Elem):
                     gl.glVertexAttribPointer(
                         attribute,
                         array.shape[-1],
-                        glutils.numpyToGLType(array.dtype),
+                        _glutils.numpyToGLType(array.dtype),
                         gl.GL_FALSE,
                         0,
                         array)
@@ -292,8 +292,11 @@ class Geometry(core.Elem):
             self.__bounds = numpy.zeros((2, 3), dtype=numpy.float32)
             # Support vertex with to 2 to 4 coordinates
             positions = self._attributes['position']
-            self.__bounds[0, :positions.shape[1]] = positions.min(axis=0)[:3]
-            self.__bounds[1, :positions.shape[1]] = positions.max(axis=0)[:3]
+            self.__bounds[0, :positions.shape[1]] = \
+                numpy.nanmin(positions, axis=0)[:3]
+            self.__bounds[1, :positions.shape[1]] = \
+                numpy.nanmax(positions, axis=0)[:3]
+            self.__bounds[numpy.isnan(self.__bounds)] = 0.  # Avoid NaNs
         return self.__bounds.copy()
 
     def prepareGL2(self, ctx):
@@ -332,7 +335,7 @@ class Geometry(core.Elem):
             with self._vbos['__indices__']:
                 gl.glDrawElements(self._MODES[self._mode],
                                   nbVertices,
-                                  glutils.numpyToGLType(self._indices.dtype),
+                                  _glutils.numpyToGLType(self._indices.dtype),
                                   ctypes.c_void_p(0))
 
 
@@ -1643,18 +1646,38 @@ class GroupDepthOffset(core.Group):
 
     def _renderGL2WithDepthRange(self, ctx):
         # gl.glDepthFunc(gl.GL_LESS)
-        for child in self.children:
-            gl.glColorMask(gl.GL_FALSE, gl.GL_FALSE, gl.GL_FALSE, gl.GL_FALSE)
-            gl.glDepthMask(gl.GL_TRUE)
-            gl.glDepthRange(self._epsilon, 1.)
+        with gl.enabled(gl.GL_CULL_FACE):
+            gl.glCullFace(gl.GL_BACK)
+            for child in self.children:
+                gl.glColorMask(
+                    gl.GL_FALSE, gl.GL_FALSE, gl.GL_FALSE, gl.GL_FALSE)
+                gl.glDepthMask(gl.GL_TRUE)
+                gl.glDepthRange(self._epsilon, 1.)
 
-            child.render(ctx)
+                child.render(ctx)
 
-            gl.glColorMask(gl.GL_TRUE, gl.GL_TRUE, gl.GL_TRUE, gl.GL_TRUE)
-            gl.glDepthMask(gl.GL_FALSE)
-            gl.glDepthRange(0., 1. - self._epsilon)
+                gl.glColorMask(
+                    gl.GL_TRUE, gl.GL_TRUE, gl.GL_TRUE, gl.GL_TRUE)
+                gl.glDepthMask(gl.GL_FALSE)
+                gl.glDepthRange(0., 1. - self._epsilon)
 
-            child.render(ctx)
+                child.render(ctx)
+
+            gl.glCullFace(gl.GL_FRONT)
+            for child in reversed(self.children):
+                gl.glColorMask(
+                    gl.GL_FALSE, gl.GL_FALSE, gl.GL_FALSE, gl.GL_FALSE)
+                gl.glDepthMask(gl.GL_TRUE)
+                gl.glDepthRange(self._epsilon, 1.)
+
+                child.render(ctx)
+
+                gl.glColorMask(
+                    gl.GL_TRUE, gl.GL_TRUE, gl.GL_TRUE, gl.GL_TRUE)
+                gl.glDepthMask(gl.GL_FALSE)
+                gl.glDepthRange(0., 1. - self._epsilon)
+
+                child.render(ctx)
 
         gl.glDepthMask(gl.GL_TRUE)
         gl.glDepthRange(0., 1.)
